import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';

/// Ícone FontAwesome.
class FAIcon extends StatelessWidget {
  /// Cria um ícone.
  /// O [size] e [color] seguem o padrão do [IconTheme] registrado.
  const FAIcon(this.icon, {Key? key, this.size, this.color, this.semanticLabel})
      : assert(icon != null),
        super(key: key);

  /// Ícone a ser exibido.
  final IconData? icon;

  /// Tamanho da fonte do ícone.
  /// Usa o valor do [IconTheme], se houver. Se o [IconTheme] não foi informado,
  /// ou não tem um valor explícito, usa-se 24.0 como padrão.
  /// Se esse [FAIcon] está sendo colocado dentro de um [IconButton], então usamos o valor de [IconButton.iconSize].
  final double? size;

  /// Cor a ser usada.
  /// Usa o valor do [IconTheme], se houver.
  final Color? color;

  /// Usado para dar nome ao ícone no modo de acessibilidade.
  final String? semanticLabel;

  @override
  Widget build(BuildContext context) {
    final IconThemeData iconTheme = IconTheme.of(context);
    final double? iconSize = size ?? iconTheme.size;

    if (icon == null) {
      return Semantics(
        label: semanticLabel,
        child: SizedBox(width: iconSize, height: iconSize),
      );
    }

    final double iconOpacity = iconTheme.opacity ?? 1.0;
    Color iconColor = color ?? iconTheme.color!;
    if (iconOpacity != 1.0) iconColor = iconColor.withOpacity(iconColor.opacity * iconOpacity);

    Widget iconWidget = RichText(
      overflow: TextOverflow.visible,
      text: TextSpan(
        text: String.fromCharCode(icon!.codePoint),
        style: TextStyle(
          inherit: false,
          color: iconColor,
          fontSize: iconSize,
          fontFamily: icon!.fontFamily,
          package: icon!.fontPackage,
        ),
      ),
    );

    return Semantics(
      label: semanticLabel,
      child: ExcludeSemantics(
        child: IntrinsicWidth(
          child: IntrinsicHeight(
            child: Center(
              child: iconWidget,
            ),
          ),
        ),
      ),
    );
  }

  @override
  void debugFillProperties(DiagnosticPropertiesBuilder properties) {
    super.debugFillProperties(properties);
    properties.add(IconDataProperty('icon', icon, ifNull: '<empty>', showName: false));
    properties.add(DoubleProperty('size', size, defaultValue: null));
    properties.add(ColorProperty('color', color, defaultValue: null));
  }
}
