import 'package:injector/injector.dart';

import '../utils/xdk_config.dart';
import '../models/ix_mobile_config.dart';
import '../models/ix_mobile_security_config.dart';
import '../storages/security_storage.dart';

/// Gerencia a autorização dos usuários.
class SecurityService {
  late IxMobileConfig _xdkConfig;
  late SecurityStorage _securityStorage;

  IxMobileSecurityConfig get _securityConfig {
    return _xdkConfig.securityConfig ?? IxMobileSecurityConfig();
  }

  SecurityService() {
    final injector = Injector.appInstance;
    _xdkConfig = getXDKConfig();
    _securityStorage = injector.get<SecurityStorage>();
  }

  /// Verifica se o usuário está logado.
  Future<bool> isLoggedIn() async {
    if (_xdkConfig.securityConfig == null) {
      return false;
    }

    if (_securityConfig.useCookieAuth) {
      return (await _securityStorage.getUserInfo()) != null;
    }

    var userToken = await _securityStorage.getUserToken();

    if (userToken != null && userToken.validUntil != null) {
      return DateTime.now().isBefore(userToken.validUntil!);
    }

    return false;
  }

  /// Verifica se o usuário logado é administrador.
  Future<bool> isAdmin() async {
    return hasClaim(_securityConfig.adminAccessClaim);
  }

  /// Verifica se o usuário tem a permissão informada.
  Future<bool> hasClaim(String claim, {isPrefix = false}) async {
    return hasAnyClaim([claim], isPrefix: isPrefix);
  }

  /// Verifica se o usuário tem alguma das claims informadas.
  Future<bool> hasAnyClaim(List<String> claims, {isPrefix = false}) async {
    var userInfo = await _securityStorage.getUserInfo();
    claims = claims.map((c) => c.toLowerCase()).toList();

    if (userInfo == null || userInfo.claims.isEmpty) {
      return false;
    }

    if (!isPrefix) {
      return userInfo.claims.any((c) => claims.contains(c.toLowerCase()));
    }

    return userInfo.claims.any((c) => claims.any((extClaim) => c.contains(extClaim)));
  }
}
