import 'dart:convert';

import '../models/user_info.dart';
import '../models/token_response.dart';

/// Ferramentas para auxiliar na segurança.
class SecurityTools {
  /// Faz o parse de um token JWT e obtém os dados do usuário e suas claims.
  static UserInfo parseUserInfo(TokenResponse tokenResponse, {bool usingIxOAuth = false, bool getClaimsFromToken = false}) {
    List<String> claims = <String>[];
    Map<String, Object> extraInfo = <String, Object>{};
    var jwtParsed = parseJwt(tokenResponse.token!);
    var excludedProperties = ['sub', 'nameid', 'email', 'nbf', 'exp', 'iat', 'iss', 'aud'];
    var ixOAuthExcludedProperties = [
      ...excludedProperties,
      'preferred_username',
      'jti',
      'allowed-origins',
      'realm_access',
      'resource_access'
    ];
    tokenResponse.validUntil = DateTime.fromMillisecondsSinceEpoch(jwtParsed['exp'] * 1000);
    UserInfo userInfo = UserInfo();
    userInfo.id = jwtParsed['sub'];

    if (!usingIxOAuth) {
      jwtParsed.forEach((key, value) {
        if (key == value) {
          claims.add(key);
        } else if (!excludedProperties.contains(key)) {
          extraInfo.putIfAbsent(key, () => value);
        }
      });

      userInfo.username = jwtParsed['nameid'];
      userInfo.email = jwtParsed['email'];
    } else {
      Map<String, String> currentClaims = getClaimsFromToken ? jwtParsed['resource_access'][jwtParsed['azp']]['roles'] : <String, String>{};

      currentClaims.forEach((key, value) => claims.add(value));
      jwtParsed.forEach((key, value) {
        if (!ixOAuthExcludedProperties.contains(key)) {
          extraInfo.putIfAbsent(key, () => value);
        }
      });

      userInfo.username = jwtParsed['preferred_username'];
    }

    userInfo.extraInfo = extraInfo;
    userInfo.claims = claims;

    return userInfo;
  }

  /// Verifica se o token contém permissões para a autenticação via OAuth da ITIX.
  static bool hasIxAuthPermissions(TokenResponse tokenResponse) {
    var jwtParsed = parseJwt(tokenResponse.token!);
    return jwtParsed.containsKey('resource_access');
  }

  /// Faz o parse de um [token] JWT.
  static Map<String, dynamic> parseJwt(String token) {
    final parts = token.split('.');

    if (parts.length != 3) {
      throw const FormatException('Token inválido.');
    }

    final payload = _decodeJwtBase64(parts[1]);
    final payloadMap = json.decode(payload);

    if (payloadMap is! Map<String, dynamic>) {
      throw const FormatException('Payload inválido.');
    }

    return payloadMap;
  }

  /// Verifica se o [token] JWT informado está expirado.
  static bool isJwtExpired(String token) {
    var expirationDate = getJwtExpiryDate(token);

    if (expirationDate != null) {
      return DateTime.now().isAfter(expirationDate);
    } else {
      return false;
    }
  }

  /// Obtém a data de expiração do [token] JWT informado.
  static DateTime? getJwtExpiryDate(String token) {
    var payload = parseJwt(token);

    if (payload['exp'] != null) {
      return DateTime.fromMillisecondsSinceEpoch(0, isUtc: true).add(Duration(seconds: payload["exp"]));
    }

    return null;
  }

  static String _decodeJwtBase64(String str) {
    var output = str.replaceAll('-', '+').replaceAll('_', '/');

    switch (output.length % 4) {
      case 0:
        break;
      case 2:
        output += '==';
        break;
      case 3:
        output += '=';
        break;
      default:
        throw Exception('Token inválido.');
    }

    return utf8.decode(base64Url.decode(output));
  }
}
