import 'dart:async';
import 'package:injector/injector.dart';
import 'package:flutter/material.dart';
import 'package:uni_links/uni_links.dart';
import 'package:url_launcher/url_launcher.dart';

import '../rest/services/auth_service.dart';
import '../rest/abstractions/base_store.dart';
import '../rest/services/request_service.dart';
import 'utils/xdk_config.dart';
import 'utils/constants.dart';
import 'models/ix_mobile_config.dart';
import 'models/message_model.dart';
import 'services/message_service.dart';
import 'services/security_service.dart';
import 'services/server_response_service.dart';
import 'storages/security_storage.dart';

/// Realiza a configuração da XDK.
class XDKSetup {
  /// Inicializa a XDK usando [config] como configuração.
  static void init(IxMobileConfig config) {
    final injector = Injector.appInstance;

    injector.clearAll();
    registerXDKConfig(config);
    injector.registerSingleton<SecurityStorage>(() => SecurityStorage());
    injector.registerSingleton<SecurityService>(() => SecurityService());
    injector.registerSingleton<RequestService>(() => RequestService());
    injector.registerSingleton<AuthService>(() => AuthService());
    injector.registerSingleton<MessageService>(() => MessageService());
    injector.registerSingleton<ServerResponseService>(() => ServerResponseService());

    // TODO: fazer dispose.
    linkStream.listen((String? link) async {
      if (link?.contains('${config.applicationId}/${Constants.oauthReturnUrl}') ?? false) {
        await _handleOAuthReturn(link!, config.context);
      }
    }, onError: (err) {
      // TODO: Tratar exceções de login e avisar ao usuário que não funcionou.
    });

    var messageService = injector.get<MessageService>();

    messageService.onNewMessage.stream.listen((messageModel) {
      showDialog(context: config.context, builder: (BuildContext context) => _getAlertDialog(messageModel, context));
    });
  }

  /// Adiciona uma lista de [stores] para consumo posterior.
  static void addStores<TStore extends BaseStore>(List<TStore> stores) {
    final injector = Injector.appInstance;

    for (var store in stores) {
      injector.registerSingleton(() => store);
    }
  }

  static AlertDialog _getAlertDialog(MessageModel messageModel, BuildContext context) {
    Widget content = Text(messageModel.message);

    if (messageModel.hasList) {
      var children = messageModel.listContent.map((item) => Text(item)).toList();
      content = SizedBox(
          width: double.maxFinite,
          child: ListView.separated(
              itemCount: children.length,
              separatorBuilder: (context, index) {
                return const Divider();
              },
              itemBuilder: (context, index) {
                return ListTile(
                  title: children[index],
                );
              }));
    }

    messageModel.actions = messageModel.actions ?? <Widget>[];
    messageModel.actions!.add(TextButton(
      onPressed: () => Navigator.pop(context, 'OK'),
      child: const Text('OK'),
    ));

    return AlertDialog(title: Text(messageModel.title ?? "Alerta"), content: content, actions: messageModel.actions);
  }

  static Future _handleOAuthReturn(String link, BuildContext context) async {
    var regex = RegExp(r'code=([^&]*)', caseSensitive: false);
    var code = regex.firstMatch(link);

    if (code != null) {
      final injector = Injector.appInstance;
      var authService = injector.get<AuthService>();
      var codeTag = code.group(0)!;
      var authCode = codeTag.replaceAll('code=', '');

      await closeWebView();
      showDialog(
        barrierDismissible: false,
        builder: (BuildContext context) => const Center(
          child: CircularProgressIndicator(
            color: Color(0xFF2850D1),
          ),
        ),
        context: context,
      );
      await authService.loadIxOAuthToken(authCode, callbackUrl: link.replaceFirst('?$codeTag', ''));
    }
  }
}
