import 'package:flutter/foundation.dart';
import 'package:injector/injector.dart';

import '../models/request_model.dart';
import '../models/store_config.dart';
import '../services/request_service.dart';

/// Classe base para os stores.
abstract class BaseStore<T> {
  /// Placeholder para o ID na URL.
  static const String urlIdPlaceholder = ":id";

  /// Configuração.
  @protected
  StoreConfig<T> config = StoreConfig<T>();

  /// Indica se a URL é absoluta (servidor externo).
  @protected
  bool isAbsoluteUrl = false;

  /// Indica se é estático (utiliza dados locais).
  @protected
  bool isStatic = false;

  @protected
  late RequestService requestService;

  @protected
  BaseStore(this.config) {
    final injector = Injector.appInstance;
    requestService = injector.get<RequestService>();
    isStatic = config.items.isNotEmpty;
    isAbsoluteUrl = _isUrlAbsolute(config.baseUrl);
  }

  /// Obtém uma URL com o controle base usando a action [action] e o
  /// [masterId] ID do registro pai, se houver (opcional).
  @protected
  String getUrl(String action, Object? masterId) {
    var baseUrl = config.baseUrl!;

    if (masterId != null) {
      baseUrl = baseUrl.replaceFirst(BaseStore.urlIdPlaceholder, masterId.toString());
    }

    return '$baseUrl${action.isNotEmpty ? '/$action' : ''}';
  }

  /// Obtém um model da requisição usando o endereço da action [action], os dados a serem enviados (opcional) [data] e o
  /// [masterId] ID do registro pai, se houver (opcional).
  @protected
  RequestModel getRequestModel(String action, {Object? data, Object? masterId}) {
    var requestModel = RequestModel();

    requestModel.url = getUrl(action, masterId);
    requestModel.isAbsolute = isAbsoluteUrl;
    requestModel.data = data;

    return requestModel;
  }

  /// Verifica se a URL [url] é absoluta.
  /// Serão consideradas URLs absolutas as que iniciarem com "[protocolo]://".
  ///     - Protocolos aceitos: http, https, ftp, ftps
  static bool _isUrlAbsolute(String? url) {
    if (url!.isEmpty) {
      return false;
    }

    var regex = RegExp(r'^(f|ht)tps?:', caseSensitive: false);

    return regex.hasMatch(url);
  }
}
