import 'package:dart_json_mapper/dart_json_mapper.dart';
import 'package:flutter/material.dart';
import 'package:http_interceptor/http/interceptor_contract.dart';
import 'package:http_interceptor/models/request_data.dart';
import 'package:http_interceptor/models/response_data.dart';
import 'package:injector/injector.dart';

import '../../core/utils/xdk_config.dart';
import '../../core/models/enums/server_response_type.dart';
import '../../core/models/bad_server_response.dart';
import '../../core/services/server_response_service.dart';
import '../../core/models/ix_mobile_config.dart';
import '../../core/storages/security_storage.dart';
import '../../core/services/message_service.dart';
import '../../core/utils/string_extensions.dart';

/// Trata os erros das requisições HTTP.
class ErrorInterceptor implements InterceptorContract {
  late IxMobileConfig _xdkConfig;
  late MessageService _messageService;
  late ServerResponseService _serverResponseService;
  late SecurityStorage _securityStorage;

  ErrorInterceptor() {
    final injector = Injector.appInstance;
    _xdkConfig = getXDKConfig();
    _securityStorage = injector.get<SecurityStorage>();
    _messageService = injector.get<MessageService>();
    _serverResponseService = injector.get<ServerResponseService>();
  }

  @override
  Future<RequestData> interceptRequest({required RequestData data}) async => data;

  @override
  Future<ResponseData> interceptResponse({required ResponseData data}) async {
    switch (data.statusCode) {
      case 401:
        _handle401();
        break;
      case 403:
        _messageService.sendError('Você não tem permissão para realizar essa ação.');
        break;
      case 500:
        _handleServerError(data);
        break;
      case 588:
        _handleBusinessExceptions(data);
        break;
    }

    return data;
  }

  void _handle401() {
    _securityStorage.clearAll();

    if (_xdkConfig.securityConfig!.useCookieAuth) {
      // TODO: goToLoginSyncPage();
    } else if (_xdkConfig.securityConfig!.useIxOAuth) {
      // TODO: abrir navegador com a página de login ITIX OAuth.
      // var url = '${_xdkConfig.baseUrl}${_xdkConfig.securityConfig!.ixOAuthLoginUrl}';
    } else {
      // TODO: goToLoginPage();
    }
  }

  /// Trata erros no servidor.
  void _handleServerError(ResponseData data) {
    var decodedResponse = JsonMapper.deserialize<BadServerResponse>(data.body);
    var errorModel = BadServerResponse(message: decodedResponse?.message ?? 'Erro genérico no servidor', logId: decodedResponse?.logId);

    _serverResponseService.setError(errorModel);

    if (_serverResponseService.shouldDisplayAlertOnError) {
      var context = _xdkConfig.context;
      var actions = <Widget>[];

      if (_xdkConfig.displayReportErrorButton && !errorModel.logId.isNullOrEmpty) {
        actions.add(TextButton(
          onPressed: () => Navigator.pop(context, 'Reportar'), // TODO:
          child: const Text('Reportar'),
        ));
      }

      _messageService.sendError(errorModel.message, title: 'Erro', actions: actions);
    }
  }

  /// Trata regras de negócios.
  void _handleBusinessExceptions(ResponseData data) {
    var decodedResponse = JsonMapper.deserialize<List<dynamic>>(data.body)!;
    var brokenRules = <String, String>{};

    for (var item in decodedResponse) {
      var map = Map<String, dynamic>.from(item);
      brokenRules[map['key']] = map['value'];
    }

    var errorMessage = BadServerResponse(
        message: 'Quebra de regra de negócios.', type: ServerResponseType.warning, isBusinessException: true, brokenRules: brokenRules);

    _serverResponseService.setError(errorMessage);

    if (_serverResponseService.shouldDisplayAlertOnError) {
      var brokenRulesAsList = <String>[];

      brokenRules.forEach((key, value) => brokenRulesAsList.add('$key: $value'));

      _messageService.sendWarning(errorMessage.message, hasList: true, list: brokenRulesAsList);
    }
  }
}
