import 'dart:async';
import 'package:injector/injector.dart';
import 'package:url_launcher/url_launcher.dart';

import '../../core/utils/xdk_config.dart';
import '../../core/utils/constants.dart';
import '../../core/models/oauth_token_response.dart';
import '../../core/models/user_info.dart';
import '../../core/models/ix_mobile_security_config.dart';
import '../../core/storages/security_storage.dart';
import '../../core/services/message_service.dart';
import '../../core/utils/security_tools.dart';
import '../../core/utils/string_extensions.dart';
import '../../core/models/ix_mobile_config.dart';
import '../../core/models/token_request.dart';
import '../../core/models/token_response.dart';
import '../models/request_model.dart';
import 'request_service.dart';

/// Gerencia a autenticação dos usuários.
class AuthService {
  /// Disparado ao logar um usuário.
  var onUserLoggedIn = StreamController<UserInfo>();

  /// Disparado ao deslogar um usuário.
  var onUserLoggedOut = StreamController<void>();

  late IxMobileConfig _xdkConfig;
  late RequestService _requestService;
  late MessageService _messageService;
  late SecurityStorage _securityStorage;

  IxMobileSecurityConfig get _securityConfig {
    return _xdkConfig.securityConfig!;
  }

  AuthService() {
    final injector = Injector.appInstance;
    _xdkConfig = getXDKConfig();
    _requestService = injector.get<RequestService>();
    _messageService = injector.get<MessageService>();
    _securityStorage = injector.get<SecurityStorage>();
  }

  /// Loga um usuário.
  Future<TokenResponse?> login(String username, String password) async {
    var tokenRequestModel = TokenRequest(username, password);
    var requestModel = RequestModel(data: tokenRequestModel);

    // Configuração de login personalizada pela aplicação cliente.
    if (_securityConfig.setupTokenRequest != null) {
      var customRequest = _securityConfig.setupTokenRequest!;
      requestModel = customRequest.setup(username, password);
    }
    // Usando entidade autorizadora.
    else if (_securityConfig.shouldUseAuthorizer) {
      // Usando entidade autorizadora externa.
      if (!_securityConfig.externalAuthUrl.isNullOrEmpty) {
        requestModel.url = _securityConfig.externalAuthUrl;
        requestModel.isAbsolute = true;
      }
      // Usando entidade autorizadora interna.
      else {
        requestModel.isBase = true;
        requestModel.url = _securityConfig.useCookieAuth ? 'auth' : 'token';
      }
    }
    // Utilizando login local.
    else {
      requestModel.url = 'auth';
      requestModel.isBase = true;
    }

    var tokenResponse = await _requestService.makePost<TokenResponse?>(requestModel);

    if (tokenResponse == null) {
      _messageService.sendWarning('Verifique os dados e tente novamente');
      return Future<TokenResponse?>.value();
    }

    if (!tokenResponse.errorMessage.isNullOrEmpty) {
      _messageService.sendError(tokenResponse.errorMessage!);
      return Future<TokenResponse?>.value();
    }

    var userInfo = SecurityTools.parseUserInfo(tokenResponse);

    await _finishLogin(tokenResponse, userInfo);
  }

  /// Finaliza a sessão do usuário.
  Future<void> logout() async {
    if (_securityConfig.useIxOAuth) {
      var refreshToken = (await _securityStorage.getUserToken())?.refreshToken ?? '';

      if (refreshToken.isNotEmpty) {
        await _requestService.makeGet<dynamic>(RequestModel(isBase: true, url: 'oauth/logout', params: {'refreshToken': refreshToken}));
      }
    }

    await _securityStorage.clearAll();
    onUserLoggedOut.add(null);
  }

  /// Faz a carga do token quando estiver usando o login via OAuth da ITIX.
  Future<void> loadIxOAuthToken(String authCode, {String? callbackUrl}) async {
    if (!_securityConfig.useIxOAuth) {
      return;
    }

    var params = <String, Object>{'authCode': authCode};

    if (!callbackUrl.isNullOrEmpty) {
      params['callbackUrl'] = callbackUrl!;
    }

    await _saveIxOAuthToken(RequestModel(isBase: true, url: 'oauth/token', params: params));
  }

  /// Obtém um novo token quando estiver usando o login via OAuth da ITIX.
  Future<void> refreshIxOAuthToken(String refreshToken) async {
    if (!_securityConfig.useIxOAuth) {
      return;
    }

    await _saveIxOAuthToken(RequestModel(isBase: true, url: 'oauth/refresh-token', params: {'refreshToken': refreshToken}));
  }

  /// Vai para a página de login.
  void goToLoginPage() {
    // TODO: redirecionar para a página de login.
  }

  /// Vai para a página de validação de cookie.
  void goToLoginSyncPage() {
    // TODO: redirecionar para a página de validação de cookie.
  }

  /// Vai para a página de trocar senha.
  void goToChangePasswordPage() {
    if (_securityConfig.useIxOAuth) {
      var url = '${_xdkConfig.baseUrl}${_securityConfig.ixOAuthChangePasswordUrl}';
      launch(url);
    }
  }

  /// Vai para a página da administração da segurança.
  void goToSecurityAdminPage() {
    if (_securityConfig.useIxOAuth) {
      var url = '${_xdkConfig.baseUrl}${_securityConfig.ixOAuthAdminUrl}';
      launch(url);
    }
  }

  /// Inicia o processo de login de acordo o tipo de autenticação.
  void startAuthProcess() {
    if (_securityConfig.useCookieAuth) {
      goToLoginSyncPage();
    } else if (_securityConfig.useIxOAuth) {
      var url = '${_xdkConfig.baseUrl}${_securityConfig.ixOAuthLoginUrl}';
      url += '?callbackUrl=ixoauth:${_xdkConfig.applicationId}/${Constants.oauthReturnUrl}';

      launch(url);
    } else {
      goToLoginPage();
    }
  }

  /// Troca a senha do usuário logado.
  Future<void> changePassword(String newPassword) async {
    await _requestService.makePost<bool>(RequestModel(url: _securityConfig.passwordChangeServerPath, data: newPassword));
  }

  Future<void> _saveIxOAuthToken(RequestModel requestModel) async {
    var tokenResponse = await _requestService.makeGet<OAuthTokenResponse>(requestModel);

    if (tokenResponse == null) {
      _messageService.sendWarning('Verifique os dados e tente novamente');
      return;
    }

    var tokenResponseModel = TokenResponse(token: tokenResponse.access_token, refreshToken: tokenResponse.refresh_token);

    if (_securityConfig.useIxOAuth && _securityConfig.getClaimsFromToken && !SecurityTools.hasIxAuthPermissions(tokenResponseModel)) {
      _messageService.sendWarning('Você não tem autorização para acessar essa funcionalidade.');
      return;
    }

    var userInfo = SecurityTools.parseUserInfo(tokenResponseModel,
        usingIxOAuth: _securityConfig.useIxOAuth, getClaimsFromToken: _securityConfig.getClaimsFromToken);

    if (_securityConfig.useIxOAuth && !_securityConfig.getClaimsFromToken) {
      var resp = await _requestService.makeGet<List<String>>(
          RequestModel(isBase: true, url: _securityConfig.ixOAuthUserInfoUrl, params: {'token': tokenResponse.access_token!}));

      if (resp == null) {
        _messageService.sendWarning('Ocorreu um erro ao obter as permissões do usuário.');
        return;
      }

      userInfo.claims = resp;

      await _finishLogin(tokenResponseModel, userInfo);
    } else {
      await _finishLogin(tokenResponseModel, userInfo);
    }
  }

  Future<void> _finishLogin(TokenResponse tokenResponse, UserInfo userInfo) async {
    await _securityStorage.saveUserToken(tokenResponse);
    await _securityStorage.saveUserInfo(userInfo);
    onUserLoggedIn.add(userInfo);
  }
}
