import 'dart:io';
import 'package:dart_json_mapper/dart_json_mapper.dart';
import 'package:http/http.dart';
import 'package:http_interceptor/http/intercepted_client.dart';

import '../../core/utils/xdk_config.dart';
import '../../core/models/ix_mobile_config.dart';
import '../../core/utils/url_tools.dart';
import '../interceptors/auth_interceptor.dart';
import '../interceptors/error_interceptor.dart';
import '../interceptors/extra_data_interceptor.dart';
import '../models/enums/http_content_type.dart';
import '../models/enums/http_method.dart';
import '../models/request_model.dart';

/// Envio de requisições.
class RequestService {
  late IxMobileConfig _xdkConfig;
  late Client _httpClient;

  RequestService() {
    var interceptors = [AuthInterceptor(), ErrorInterceptor(), ExtraDataInterceptor()];
    _xdkConfig = getXDKConfig();
    _httpClient = InterceptedClient.build(interceptors: [...interceptors, ..._xdkConfig.interceptors]);
  }

  /// Cria uma requisição HTTP do tipo GET com a [config] informada.
  Future<T?> makeGet<T>(RequestModel? config) async {
    config = config ?? RequestModel();
    return _makeRequest<T>(HttpMethod.get, _getUrl(config), config.headers,
        data: config.data, httpParams: config.params, contentType: config.contentType, responseType: config.responseType);
  }

  /// Cria uma requisição HTTP do tipo POST com a [config] informada
  Future<T?> makePost<T>(RequestModel? config) async {
    config = config ?? RequestModel();
    return _makeRequest<T>(HttpMethod.post, _getUrl(config), config.headers,
        data: config.data, httpParams: config.params, contentType: config.contentType, responseType: config.responseType);
  }

  /// Cria uma requisição HTTP do tipo PUT com a [config] informada
  Future<T?> makePut<T>(RequestModel? config) async {
    config = config ?? RequestModel();
    return _makeRequest<T>(HttpMethod.put, _getUrl(config), config.headers, data: config.data);
  }

  /// Cria uma requisição HTTP do tipo DELETE com a [config] informada
  Future<bool> makeDelete(RequestModel? config) async {
    config = config ?? RequestModel();
    var resp = await _makeRequest<bool>(HttpMethod.delete, _getUrl(config), config.headers);

    return resp ?? false;
  }

  Future<T?> _makeRequest<T>(HttpMethod type, String url, Map<String, String> headers,
      {Object? data, Map<String, Object>? httpParams, String contentType = HttpContentType.applicationJson, String? responseType}) async {
    Future<Response> request;
    var queryParams = httpParams?.isNotEmpty ?? false ? "?" : "";

    httpParams?.forEach((key, value) {
      queryParams += (queryParams.length > 1 ? "&" : "") + "$key=$value";
    });

    var parsedUrl = Uri.parse("$url$queryParams");
    var bodyString = data != null ? JsonMapper.serialize(data) : "";
    var headerItems = <String, String>{};

    if (contentType != HttpContentType.empty) {
      headerItems[HttpHeaders.contentTypeHeader] = contentType;
    }

    if (responseType?.isNotEmpty ?? false) {
      headerItems[HttpHeaders.acceptHeader] = responseType!;
    }

    headers.forEach((key, value) {
      headerItems[key] = value;
    });

    if (contentType == HttpContentType.empty) {
      request = _httpClient.post(parsedUrl, headers: headerItems, body: data);
    } else if (contentType == HttpContentType.applicationFormEncoded) {
      request = _httpClient.post(parsedUrl, headers: headerItems, body: data);
    } else {
      switch (type) {
        case HttpMethod.get:
          request = _httpClient.get(parsedUrl, headers: headerItems);
          break;
        case HttpMethod.put:
          request = _httpClient.put(parsedUrl, headers: headerItems, body: bodyString);
          break;
        case HttpMethod.post:
          request = _httpClient.post(parsedUrl, headers: headerItems, body: bodyString);
          break;
        case HttpMethod.delete:
          request = _httpClient.delete(parsedUrl, headers: headerItems);
          break;
      }
    }

    var requestResponse = await request;

    if (requestResponse.statusCode == 200) {
      var decodedResponse = JsonMapper.deserialize<T>(requestResponse.body);
      return decodedResponse!;
    }

    return null;
  }

  String _getUrl(RequestModel requestModel) {
    return requestModel.isAbsolute
        ? requestModel.url ?? ""
        : UrlTools.getUrl(requestModel.url ?? "", _xdkConfig.baseUrl ?? "",
            isBase: requestModel.isBase,
            id: requestModel.id,
            version: (requestModel.version?.isEmpty ?? true) ? _xdkConfig.defaultApiVersion : requestModel.version);
  }
}
